function ZhLocation(ZHPromise, ScreenService) {

	var defer = false;
	var timers = {};
	var self = this;

	// Browser navigator ignores the providerName
	var browserOptions = {
		enableHighAccuracy: true,
		timeout: 10000,
		maximumAge: 0
	};
	var browserNavigator = {
		isAvailableProvider: function() {
			return "geolocation" in navigator;
		},
		isProviderEnabled: this.isAvailableProvider,
		getCurrentPosition: function() {
			function errorHandler(error) {
				rejectPromise(error.message);
			}
			if (!this.isAvailableProvider()) {
				rejectPromise('Geolocation is not supported by your browser');
			} else {
				navigator.geolocation.getCurrentPosition(self.changePosition, errorHandler, browserOptions);
			}
		},
		watchPosition: function(providerName, changeCallback, errorCallback) {
			return navigator.geolocation.watchPosition(changeCallback, errorCallback, browserOptions);
		},
		clearWatch: function(watchId) {
			return navigator.geolocation.clearWatch(watchId);
		}
	};

	var locationInterface = getLocationInterface();

	function getLocationInterface() {
		return window.ZhNativeInterface ? window.ZhNativeInterface : browserNavigator;
	}

	function rejectPromise(message) {
		defer.reject(ScreenService.i18n(message));
		defer = false;
	}

	this.changePosition = function(position) {
		position = Util.isObject(position) ? position : JSON.parse(position);
		defer.resolve(position);
		defer = false;
	};

	this.changeWatchPosition = function(position, watchId) {
		position = JSON.parse(position);
		if (watchId && timers[watchId]) {
			timers[watchId].changeCallback(position);
		}
	};

	this.isAvailableProvider = function(providerName) {
		return locationInterface.isAvailableProvider(providerName);
	};

	this.isProviderEnabled = function(providerName) {
		return locationInterface.isProviderEnabled(providerName);
	};

	this.getCurrentPosition = function(providerName) {
		if (!defer) {
			defer = ZHPromise.defer();
		}
		locationInterface.getCurrentPosition(providerName);
		return defer.promise;
	};

	this.watchPosition = function(providerName, changeCallback, errorCallback) {
		if (!locationInterface.isAvailableProvider(providerName)) {
			return false;
		}
		var watchId = locationInterface.watchPosition(providerName, changeCallback, errorCallback);
		timers[watchId] = {
			providerName: providerName,
			changeCallback: changeCallback,
			errorCallback: errorCallback
		};
		return watchId;
	};

	this.clearWatch = function(watchId) {
		if (watchId && timers[watchId]) {
			locationInterface.clearWatch(watchId);
			delete timers[watchId];
		}
	};
}

ZhLocation.GPS = 'GPS_PROVIDER';
ZhLocation.NETWORK = 'NETWORK_PROVIDER';
ZhLocation.PASSIVE = 'PASSIVE_PROVIDER';